// cargo run -- --script test_vectors/fizz_buzz.lua

mod instruction;
mod loader;
mod state;
mod value;

#[cfg (test)]
mod tests;

fn main () {
	use state::State;
	
	let mut list_bytecode = false;
	let mut pipe_bytecode = false;
	let mut script = None;
	
	let mut args = std::env::args ();
	let exe_name = args.next ().unwrap ();
	
	while let Some (arg) = args.next () {
		match arg.as_str () {
			"--list-bytecode" => list_bytecode = true,
			"--pipe-bytecode" => pipe_bytecode = true,
			"--script" => script = Some (args.next ().unwrap ()),
			"--" => break,
			_ => panic! ("can't parse args"),
		}
	}
	
	let chunk = if let Some (script) = script {
		let bytecode = loader::compile_bytecode_from_file (&script);
		dbg! (&bytecode [0..48]);
		let mut rdr = std::io::Cursor::new (bytecode);
		loader::parse_chunk (&mut rdr).unwrap ()
	}
	else if pipe_bytecode {
		let mut stdin = std::io::stdin ().lock ();
		loader::parse_chunk (&mut stdin).unwrap ()
	}
	else {
		unimplemented!();
	};
	
	if list_bytecode {
		dbg! (&chunk);
	}
	
	let mut vm = State::default ();
	if std::env::var("LUA_DEBUG").is_ok() {
		vm.debug_print = true;
	}
	
	let upvalues = State::upvalues_from_args ([exe_name].into_iter ().chain (args));
	
	vm.breakpoints.push (state::Breakpoint {
		block_idx: 3,
		program_counter: 0,
	});
	
	vm.execute_chunk (&chunk, &upvalues);
}
